Public Class ColorPalette
    'Public events
    Public Event LeftClick(ByVal sender As Object, ByVal e As EventArgs)
    Public Event RightClick(ByVal sender As Object, ByVal e As EventArgs)

    'Public members
    Public Buttons As New ArrayList()
    Public ButtonSize As Integer = 15
    Public ButtonSpacing As Integer = 5
    Public LeftColor As Color = Color.Black
    Public RightColor As Color = Color.White

    'Private members
    Private LeftButton As ColorPaletteButton
    Private RightButton As ColorPaletteButton

    'Add a new color button to the control
    Public Sub AddColor(ByVal newColor As Color)
        'Create the button
        Dim objColorPaletteButton As New ColorPaletteButton(newColor)

        'Add it to the list
        Buttons.Add(objColorPaletteButton)

        'Do we have a button assigned to the left button yet?
        If LeftButton Is Nothing Then

            objColorPaletteButton.ButtonAssignment = _
               ColorPaletteButton.ButtonAssignments.LeftButton

            LeftButton = objColorPaletteButton

        ElseIf RightButton Is Nothing Then  'How about the right button?

            objColorPaletteButton.ButtonAssignment = _
                 ColorPaletteButton.ButtonAssignments.RightButton

            RightButton = objColorPaletteButton

        End If
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        'Add the colors
        AddColor(Color.Black)
        AddColor(Color.White)
        AddColor(Color.Red)
        AddColor(Color.Blue)
        AddColor(Color.Green)
        AddColor(Color.Gray)
        AddColor(Color.DarkRed)
        AddColor(Color.DarkBlue)
        AddColor(Color.DarkGreen)
        AddColor(Color.DarkGray)
        AddColor(Color.FromArgb(208, 112, 222))
        AddColor(Drawing.SystemColors.MenuBar)
    End Sub

    Private Sub ColorPalette_MouseUp(ByVal sender As Object, _
        ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp

        'Find the button that we clicked
        Dim objColorPaletteButton As ColorPaletteButton = GetButtonAt(e.X, e.Y)

        If Not objColorPaletteButton Is Nothing Then

            'Was the left button clicked
            If e.Button = MouseButtons.Left Then

                'Make sure that this button is not the current right button
                If objColorPaletteButton IsNot RightButton Then

                    'Set the color
                    LeftColor = objColorPaletteButton.Color

                    'Clear the existing selection.
                    If LeftButton IsNot Nothing Then

                        LeftButton.ButtonAssignment = _
                            ColorPaletteButton.ButtonAssignments.None

                        Me.Invalidate(LeftButton.Rectangle)

                    End If

                    'Mark the button
                    objColorPaletteButton.ButtonAssignment = _
                         ColorPaletteButton.ButtonAssignments.LeftButton

                    Me.Invalidate(objColorPaletteButton.Rectangle)

                    LeftButton = objColorPaletteButton

                    'Raise the event
                    RaiseEvent LeftClick(Me, New EventArgs())

                End If

            ElseIf e.Button = MouseButtons.Right Then

                'Make sure this button is not the current left button
                If objColorPaletteButton IsNot LeftButton Then

                    'Set the color
                    RightColor = objColorPaletteButton.Color

                    'Clear the existing selection
                    If RightButton IsNot Nothing Then

                        RightButton.ButtonAssignment = _
                            ColorPaletteButton.ButtonAssignments.None

                        Me.Invalidate(RightButton.Rectangle)

                    End If

                    'Mark the button
                    objColorPaletteButton.ButtonAssignment = _
                         ColorPaletteButton.ButtonAssignments.RightButton

                    Me.Invalidate(objColorPaletteButton.Rectangle)

                    RightButton = objColorPaletteButton

                    'Raise the event
                    RaiseEvent RightClick(Me, New EventArgs())

                End If

            End If

        Else
            'Display the color dialog
            If dlgColor.ShowDialog = DialogResult.OK Then

                'Add the new color
                AddColor(dlgColor.Color)

                'Resize the palette to show the dialog
                OnResize(New EventArgs())

            End If

        End If
    End Sub

    Private Sub ColorPalette_Paint(ByVal sender As Object, _
        ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint

        'Loop through the buttons
        For Each objColorPaletteButton As ColorPaletteButton In Buttons

            'Do we need to draw?
            If e.ClipRectangle.IntersectsWith(objColorPaletteButton.Rectangle) Then
                objColorPaletteButton.Draw(e.Graphics)
            End If

        Next
    End Sub

    Private Sub ColorPalette_Resize(ByVal sender As Object, _
        ByVal e As System.EventArgs) Handles Me.Resize

        'Declare variables to hold the position
        Dim intX As Integer
        Dim intY As Integer

        'Go through the array and position the buttons
        For Each objColorPaletteButton As ColorPaletteButton In Buttons

            'Position the button
            objColorPaletteButton.SetPosition(intX, intY, ButtonSize)

            'Move to the next one
            intX += (ButtonSize + ButtonSpacing)

            'Do we need to go down to the next row
            If intX + ButtonSize > Width Then

                'Move y
                intY += (ButtonSize + ButtonSpacing)

                'Reset x
                intX = 0

            End If

        Next

        'Redraw
        Me.Invalidate()
    End Sub

    Public Function GetButtonAt(ByVal x As Integer, ByVal y As Integer) _
        As ColorPaletteButton

        'Set the default return value
        GetButtonAt = Nothing

        'Go through each button in the collection
        For Each objColorPaletteButton As ColorPaletteButton In Buttons
            'Is this button in the rectangle?
            If objColorPaletteButton.Rectangle.Contains(x, y) Then
                Return objColorPaletteButton
            End If
        Next
    End Function
End Class
