Public Class PaintCanvas
    'Public enumerations
    Public Enum GraphicTools As Integer
        CirclePen = 0
        HollowCirclePen = 1
    End Enum

    Public Enum GraphicSizes As Integer
        Small = 4
        Medium = 10
        Large = 20
    End Enum

    'Public members
    Public GraphicsItems As New ArrayList()
    Public GraphicTool As GraphicTools = GraphicTools.CirclePen
    Public GraphicSize As GraphicSizes = GraphicSizes.Medium

    Public GraphicLeftColor As Color = Color.Black
    Public GraphicRightColor As Color = Color.White

    Private Sub DoMousePaint(ByVal e As MouseEventArgs)
        'Store the new item somewhere
        Dim objGraphicsItem As GraphicsItem

        'What color do we want to use?
        Dim objColor As Color = GraphicLeftColor

        If e.Button = MouseButtons.Right Then
            objColor = GraphicRightColor
        End If

        'What tool are you using?
        Select Case GraphicTool

            'CirclePen
            Case GraphicTools.CirclePen

                'Create a new graphics circle
                Dim objGraphicsCircle As New GraphicsCircle()

                'Set the point for drawing
                objGraphicsCircle.SetPoint(e.X, e.Y, GraphicSize, _
                    objColor, True)

                'Store this for addition
                objGraphicsItem = objGraphicsCircle

                'HollowCirclePen
            Case GraphicTools.HollowCirclePen

                'Create a new graphics circle
                Dim objGraphicsCircle As New GraphicsCircle()

                'Set the point for drawing
                objGraphicsCircle.SetPoint(e.X, e.Y, GraphicSize, _
                    objColor, False)

                'Store this for addition
                objGraphicsItem = objGraphicsCircle

        End Select

        'Were you given an item?
        If objGraphicsItem IsNot Nothing Then

            'Add it to the list
            GraphicsItems.Add(objGraphicsItem)

            'Invalidate the Control
            Me.Invalidate(objGraphicsItem.Rectangle)

        End If
    End Sub

    Private Sub PaintCanvas_MouseDown(ByVal sender As Object, _
        ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown

        'Is the left or right mouse button down?
        If e.Button = MouseButtons.Left Or e.Button = MouseButtons.Right Then
            DoMousePaint(e)
        End If
    End Sub

    Private Sub PaintCanvas_MouseMove(ByVal sender As Object, _
        ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove

        'Is the left or right mouse button down?
        If e.Button = MouseButtons.Left Or e.Button = MouseButtons.Right Then
            DoMousePaint(e)
        End If
    End Sub

    Private Sub PaintCanvas_Paint(ByVal sender As Object, _
        ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint

        'Go through the list
        For Each objGraphicsItem As GraphicsItem In GraphicsItems
            'Do we need to be drawn?
            If e.ClipRectangle.IntersectsWith(objGraphicsItem.Rectangle) Then
                'Ask each item to draw itself
                objGraphicsItem.Draw(e.Graphics)
            End If
        Next
    End Sub

    Protected Overrides Sub OnPaintBackground( _
        ByVal e As System.Windows.Forms.PaintEventArgs)

        'Paint the invalid region with the background brush
        Dim backgroundBrush As New SolidBrush(BackColor)
        e.Graphics.FillRectangle(backgroundBrush, e.ClipRectangle)

        'Paint the image
        If Not BackgroundImage Is Nothing Then

            'Find our client rectangle
            Dim clientRectangle As New Rectangle(0, 0, Width, Height)

            'How big is the image?
            Dim intImageWidth As Integer = BackgroundImage.Width
            Dim intImageHeight As Integer = BackgroundImage.Height

            'Whats the aspect ratio?
            Dim ratio As Double = _
                CType(intImageHeight, Double) / CType(intImageWidth, Double)

            'Scale the image
            If intImageWidth > clientRectangle.Width Then
                intImageWidth = clientRectangle.Width
                intImageHeight = _
                    CType(CType(intImageWidth, Double) * ratio, Integer)
            End If

            If intImageHeight > clientRectangle.Height Then
                intImageHeight = clientRectangle.Height
                intImageWidth = _
                    CType(CType(intImageHeight, Double) / ratio, Integer)
            End If

            'You need to center the image
            Dim pntImageLocation As New Point( _
                (clientRectangle.Width / 2) - (intImageWidth / 2), _
                (clientRectangle.Height / 2) - (intImageHeight / 2))
            Dim sizImageSize As New Size(intImageWidth, intImageHeight)
            Dim recImageRectangle As New Rectangle(pntImageLocation, sizImageSize)

            'Draw the image
            e.Graphics.DrawImage(BackgroundImage, clientRectangle)

        End If
    End Sub

    Private Sub PaintCanvas_Resize(ByVal sender As Object, _
        ByVal e As System.EventArgs) Handles Me.Resize

        'Invalidate the control
        Me.Invalidate()
    End Sub
End Class
